set more off
set linesize 240
clear all
capture log close
log using ch14_small.txt,text replace
*************************************************************
* ch14_1.do                    
* replication files for Stock-Watson 4th Editions Chapter 14
************************************************************ 
* Read In Data 
* (Note: Change path name so that it is appropriate for your computer);
use "/Users/mwatson/Dropbox/TB/4E/WebSupplements/ForStudents/SW_4E_ReplicationFiles/SW_4E_Replication_Data/ca_school_testscore_insample.dta"
append using "/Users/mwatson/Dropbox/TB/4E/WebSupplements/ForStudents/SW_4E_ReplicationFiles/SW_4E_Replication_Data/ca_school_testscore_outofsample.dta"

*** Insample and outofsample indicators
generate insample = _n <= 1966
generate outofsample = _n > 1966
*****
desc
**** Small Data Set Prediction
global pred_1 "str_s med_income_z te_avgyr_s exp_1000_1999_d" 

* In-sample (raw data)
local i = 0
foreach x in $pred_1{
	local i = `i'+1
	generate x_`i'= `x'
	quiet sum x_`i' if insample
	generate s_x_`i' = (x_`i'-r(mean))/r(sd)
}
*** s_x_i contains data standardized using in-sample values
quiet summarize testscore if insample
generate testscore_dm = testscore-r(mean)

global pred_x "x_*" 
global pred_sx = "s_x_*" 

* Compute Cross-fold RMSE using regression in-sample
* Only doing OLS prediction, so no need to used standardized values
regress testscore $pred_x if insample
predict testscore_ols
generate e_ols = testscore-testscore_ols
generate e_ols2 = e_ols^2
quiet sum e_ols2 if outofsample
scalar rmse_oos_ols = sqrt(r(mean))
***** Compute in-sample RMSE by Cross-validation
** This uses the ADO file crossfold.ado .. it computes RMSE for each of k-folds .. stored in the vector r(est). I compute overall RMSE by averaging RMSEs and taking square root
crossfold regress testscore $pred_x if insample, k(10) 
matrix a = r(est)'*r(est)
scalar rmse_cv_is_ols = sqrt(a[1,1]/10)
*** Print Results
display "RMSE In-Sample by 10-fold CV = " rmse_cv_is_ols
display "RMSE Out-of-Sample = " rmse_oos_ols
*
* Compute Regression using standardized regressors (Results shown in Table 14.4)
regress testscore_dm $pred_sx if insample, noconstant


log close